/*
 *
 * stars3d.c v0.5 - 2005-01-10 - MinGW32 support.
 *           v0.1 - 2000-09-16
 *
 * Clemens Fuchslocher <clemens@allesdurcheinander.de>
 *
 */

/*
 *
 * SDL - Simple DirectMedia Layer <http://www.libsdl.org/>
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <math.h>

#include <SDL.h>

#ifdef __MINGW32__
#define SDL_main main
#endif

#define STARS   800
#define SPEED   1
#define DEEP    256
#define FPS	1000/60

typedef struct Star_tag
{
	Sint32 x, y, z;
} Star;

void new_star (Star *star, SDL_Surface *surface);
void put_pixel (Uint32 x, Uint32 y, Uint32 color, SDL_Surface *surface);

int main (int argc, char *argv[])
{
	SDL_Surface *screen;

	Uint32 i, frames = 0;

	Star stars[STARS];
	Sint32 x, y;
	Uint32 color;

	Uint32 this_tick_count;
	Uint32 last_tick_count;

	time_t runtime = time (NULL);
	srand (time (NULL));

#ifdef __MINGW32__
	FILE *fps;
#endif

	if (SDL_Init (SDL_INIT_VIDEO) < 0)
	{
		fprintf (stderr, ".oO ( Oops : %s )\n", SDL_GetError ());
		exit (EXIT_FAILURE);
	}
	atexit (SDL_Quit);

	for (i = 0; i < SDL_NUMEVENTS; i++)
	{
		if ((i != SDL_KEYDOWN) && (i != SDL_QUIT))
		{
			SDL_EventState(i, SDL_IGNORE);
		}
	}

	screen = SDL_SetVideoMode (800, 600, 32, SDL_HWSURFACE | SDL_FULLSCREEN | SDL_DOUBLEBUF);
	if (screen == NULL)
	{
		fprintf (stderr, ".oO ( Oops: %s )\n", SDL_GetError ());
		exit (EXIT_FAILURE);
	}

	SDL_WM_SetCaption ("stars3d", "stars3d");
	SDL_ShowCursor (0);

	for (i = 0; i < STARS; i++)
	{
		new_star (&stars[i], screen);
	}

	last_tick_count = SDL_GetTicks ();
	while (SDL_PollEvent (NULL) == 0)
	{
		frames++;
		if (SDL_MUSTLOCK (screen))
		{
			if (SDL_LockSurface (screen) < 0)
			{
				fprintf (stderr, ".oO ( Oops: %s )\n", SDL_GetError ());
				exit (EXIT_FAILURE);
			}
		}

		for (i = 0; i < STARS; i++)
		{
			stars[i].z = stars[i].z - SPEED;
			if (stars[i].z < 1)
			{
				new_star (&stars[i], screen);
				stars[i].z = DEEP;
			}

			/* 3d -> 2d projektion (mc extra 3/95) */
			x = stars[i].x / stars[i].z + screen->w/2;
			y = stars[i].y / stars[i].z + screen->h/2;

			color = (DEEP - stars[i].z);
			color = (color << screen->format->Rshift) |
				(color << screen->format->Gshift) |
				(color << screen->format->Bshift);

			put_pixel (x,   y,   color, screen);
			put_pixel (x,   y+1, color, screen);
			put_pixel (x,   y-1, color, screen);
			put_pixel (x+1, y,   color, screen);
			put_pixel (x-1, y,   color, screen);
		}

		if (SDL_MUSTLOCK (screen))
		{
			SDL_UnlockSurface (screen);
		}

		this_tick_count = SDL_GetTicks ();
		if ((this_tick_count - last_tick_count) < FPS)
		{
			SDL_Delay (FPS - (this_tick_count - last_tick_count));
		}
		last_tick_count = this_tick_count;

		SDL_Flip (screen);
		SDL_FillRect (screen, NULL, 0);
	}

#ifdef __MINGW32__
	fps = fopen ("fps.txt", "w");
	if (fps != NULL)
	{
		fprintf (fps, "fps    : %ld\n", frames / (time (NULL) - runtime));
		fprintf (fps, "runtime: %ld\n", time (NULL) - runtime);
		fclose (fps);
	}
#else
	printf ("fps    : %ld\n", frames / (time (NULL) - runtime));
	printf ("runtime: %ld\n", time (NULL) - runtime);
#endif

	SDL_ShowCursor (1);
	SDL_Quit ();

	return EXIT_SUCCESS;
}

void new_star (Star *star, SDL_Surface *surface)
{
	star->x = ((rand () % surface->w) - surface->w/2) * 100;
	star->y = ((rand () % surface->h) - surface->h/2) * 100;
	star->z = ((rand () % DEEP)) + 1;
}

void put_pixel (Uint32 x, Uint32 y, Uint32 color, SDL_Surface *surface)
{
	if (x < surface->w)
	{
		if (y < surface->h)
		{
			*((Uint32 *) surface->pixels + (y * surface->pitch/4 + x)) = color;
		}
	}
}

